// Copyright 2019 RnD Center "ELVEES", JSC

#include "cvatparse.h"

#include <tinyxml2.h>

#include <cstring>
#include <iostream>
#include <string>

#include "utils.h"

bool cvatparse::load(const std::string &path, annotations &ann) {
    tinyxml2::XMLDocument xml_doc;
    if (!load_xml(xml_doc, path)) {
        return false;
    }
    tinyxml2::XMLNode *pannotations = xml_doc.RootElement();
    if (pannotations == nullptr) {
        std::cout << "Incorrect data in file " << path << std::endl;
        return false;
    }
    tinyxml2::XMLElement *pannotations_element = pannotations->FirstChildElement("image");
    if (pannotations_element != nullptr) {
        load_elems(pannotations_element, ann.images);
    } else {
        pannotations_element = pannotations->FirstChildElement("track");
        if (pannotations_element != nullptr) {
                    load_elems(pannotations_element, ann.tracks);
        } else {
            return false;
        }
    }
    pannotations_element = pannotations->FirstChildElement("alg-info");
    if (pannotations_element != nullptr) {
        ann.alg_info = parse_attr(pannotations_element);
    }
    return true;
}

bool cvatparse::dump(const std::string &path, annotations &ann) {
    tinyxml2::XMLDocument xml_doc;
    tinyxml2::XMLNode *pannotations = xml_doc.NewElement("annotations");
    xml_doc.InsertEndChild(pannotations);
    if (!ann.images.empty()) {
        dump_elems(xml_doc, pannotations, ann.images);
    } else if (!ann.tracks.empty()) {
        dump_elems(xml_doc, pannotations, ann.tracks);
    } else {
        return false;
    }
    if (!ann.alg_info.empty()) {
        tinyxml2::XMLElement *palg_info = xml_doc.NewElement("alg-info");
        serialize_attr(xml_doc, palg_info, ann.alg_info);
        pannotations->InsertEndChild(palg_info);
    }
    tinyxml2::XMLError file_saved = xml_doc.SaveFile(path.c_str());
    if (file_saved != tinyxml2::XML_SUCCESS) {
        return false;
    }
    return true;
}
